﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace CameraMoveExample
{
    class WorldObject
    {
        public WorldObject Parent { get; set; }
        public Vector2 PositionInParentSpace { get; set; }
        public float RotationInParentSpace { get; set; }

        public Texture2D Texture;
        public Vector2 TextureOrigin;


        public WorldObject(Texture2D texture)
        {
            Texture = texture;
            TextureOrigin = new Vector2(texture.Width, texture.Height) / 2;
            Parent = null;
            PositionInParentSpace = Vector2.Zero;
            RotationInParentSpace = 0.0f;
        }

        public WorldObject(Texture2D texture, WorldObject parent)
        {
            Texture = texture;
            TextureOrigin = new Vector2(texture.Width, texture.Height) / 2;
            Parent = parent;
            PositionInParentSpace = Vector2.Zero;
            RotationInParentSpace = 0.0f;
        }



        public Vector2 rotate(Vector2 vec, float angle)
        {
            return new Vector2(vec.X * (float)Math.Cos(angle) -
                                        vec.Y * (float)Math.Sin(angle),
                                        vec.X * (float)Math.Sin(angle) +
                                        vec.Y * (float)Math.Cos(angle));
        }

        public Vector2 wordToLocal(Vector2 worldPos, WorldObject obj)
        {
            if (obj == null)
            {
                return worldPos;
            }
            Vector2 parentPos = wordToLocal(worldPos, obj.Parent);
            Vector2 localPos = rotate(parentPos - obj.PositionInParentSpace, -obj.RotationInParentSpace);
            return localPos;
        }

        public Vector2 PositionInWorldSpace
        {
            get
            {
                Vector2 worldPosition = PositionInParentSpace;
                WorldObject parent = Parent;
                while (parent != null)
                {
                    worldPosition = rotate(worldPosition, parent.RotationInParentSpace);
                    worldPosition = worldPosition + parent.PositionInParentSpace;
                    parent = parent.Parent;
                }
                return worldPosition;
            }
            set
            {
                PositionInParentSpace = wordToLocal(value, Parent);
            }
        }

        public float RotationInWorldSpace
        {
            get
            {
                // Get the rotation of the object in world space
                float rotation = RotationInParentSpace;
                WorldObject parent = Parent;
                while (parent != null)
                {
                    rotation = rotation + parent.RotationInParentSpace;
                    parent = parent.Parent;
                }
                return rotation;
            }
            set
            {
                float ancestorRotation = 0.0f;
                WorldObject parent = Parent;
                while (parent != null)
                {
                    ancestorRotation = ancestorRotation + parent.RotationInParentSpace;
                    parent = parent.Parent;
                }
                RotationInParentSpace = value - ancestorRotation;
            }
        }
    }
}
